@@ -10,13 +10,13 @@ const expect = chai.expect;
 
 chai.config.includeStack = true;
 
-describe('MailComposer unit tests', function() {
-    it('should create new MailComposer', function() {
+describe('MailComposer unit tests', function () {
+    it('should create new MailComposer', function () {
         expect(new MailComposer({})).to.exist;
     });
 
-    describe('#compile', function() {
-        it('should use Mixed structure with text and attachment', function() {
+    describe('#compile', function () {
+        it('should use Mixed structure with text and attachment', function () {
             let data = {
                 text: 'abc',
                 attachments: [
@@ -33,7 +33,7 @@ describe('MailComposer unit tests', function() {
             compiler._createMixed.restore();
         });
 
-        it('should use Mixed structure with multiple attachments', function() {
+        it('should use Mixed structure with multiple attachments', function () {
             let data = {
                 attachments: [
                     {
@@ -52,7 +52,7 @@ describe('MailComposer unit tests', function() {
             compiler._createMixed.restore();
         });
 
-        it('should create Alternative structure with text and html', function() {
+        it('should create Alternative structure with text and html', function () {
             let data = {
                 text: 'abc',
                 html: 'def'
@@ -64,13 +64,13 @@ describe('MailComposer unit tests', function() {
             expect(compiler._createAlternative.callCount).to.equal(1);
 
             expect(compiler._alternatives.length).to.equal(2);
-            expect(compiler._alternatives[0].contentType).to.equal('text/plain');
-            expect(compiler._alternatives[1].contentType).to.equal('text/html');
+            expect(compiler._alternatives[0].contentType).to.equal('text/plain; charset=utf-8');
+            expect(compiler._alternatives[1].contentType).to.equal('text/html; charset=utf-8');
 
             compiler._createAlternative.restore();
         });
 
-        it('should create Alternative structure with text, watchHtml and html', function() {
+        it('should create Alternative structure with text, watchHtml and html', function () {
             let data = {
                 text: 'abc',
                 html: 'def',
@@ -82,13 +82,13 @@ describe('MailComposer unit tests', function() {
             compiler.compile();
             expect(compiler._createAlternative.callCount).to.equal(1);
             expect(compiler._alternatives.length).to.equal(3);
-            expect(compiler._alternatives[0].contentType).to.equal('text/plain');
-            expect(compiler._alternatives[1].contentType).to.equal('text/watch-html');
-            expect(compiler._alternatives[2].contentType).to.equal('text/html');
+            expect(compiler._alternatives[0].contentType).to.equal('text/plain; charset=utf-8');
+            expect(compiler._alternatives[1].contentType).to.equal('text/watch-html; charset=utf-8');
+            expect(compiler._alternatives[2].contentType).to.equal('text/html; charset=utf-8');
             compiler._createAlternative.restore();
         });
 
-        it('should create Alternative structure with text, amp and html', function() {
+        it('should create Alternative structure with text, amp and html', function () {
             let data = {
                 text: 'abc',
                 html: 'def',
@@ -100,13 +100,13 @@ describe('MailComposer unit tests', function() {
             compiler.compile();
             expect(compiler._createAlternative.callCount).to.equal(1);
             expect(compiler._alternatives.length).to.equal(3);
-            expect(compiler._alternatives[0].contentType).to.equal('text/plain');
-            expect(compiler._alternatives[1].contentType).to.equal('text/x-amp-html');
-            expect(compiler._alternatives[2].contentType).to.equal('text/html');
+            expect(compiler._alternatives[0].contentType).to.equal('text/plain; charset=utf-8');
+            expect(compiler._alternatives[1].contentType).to.equal('text/x-amp-html; charset=utf-8');
+            expect(compiler._alternatives[2].contentType).to.equal('text/html; charset=utf-8');
             compiler._createAlternative.restore();
         });
 
-        it('should create Alternative structure with text, icalEvent and html', function() {
+        it('should create Alternative structure with text, icalEvent and html', function () {
             let data = {
                 text: 'abc',
                 html: 'def',
@@ -118,13 +118,13 @@ describe('MailComposer unit tests', function() {
             compiler.compile();
             expect(compiler._createAlternative.callCount).to.equal(1);
             expect(compiler._alternatives.length).to.equal(3);
-            expect(compiler._alternatives[0].contentType).to.equal('text/plain');
-            expect(compiler._alternatives[1].contentType).to.equal('text/html');
-            expect(compiler._alternatives[2].contentType).to.equal('text/calendar; charset="utf-8"; method=PUBLISH');
+            expect(compiler._alternatives[0].contentType).to.equal('text/plain; charset=utf-8');
+            expect(compiler._alternatives[1].contentType).to.equal('text/html; charset=utf-8');
+            expect(compiler._alternatives[2].contentType).to.equal('text/calendar; charset=utf-8; method=PUBLISH');
             compiler._createAlternative.restore();
         });
 
-        it('should create Alternative structure using encoded icalEvent', function(done) {
+        it('should create Alternative structure using encoded icalEvent', function (done) {
             let data = {
                 text: 'abc',
                 html: 'def',
@@ -138,7 +138,7 @@ describe('MailComposer unit tests', function() {
 
             let compiler = new MailComposer(data);
             sinon.spy(compiler, '_createAlternative');
-            compiler.compile().build(function(err, message) {
+            compiler.compile().build(function (err, message) {
                 expect(err).to.not.exist;
                 let msg = message.toString();
                 expect(
@@ -153,7 +153,7 @@ describe('MailComposer unit tests', function() {
             });
         });
 
-        it('should create Alternative structure with text, html and cid attachment', function() {
+        it('should create Alternative structure with text, html and cid attachment', function () {
             let data = {
                 text: 'abc',
                 html: 'def',
@@ -176,7 +176,7 @@ describe('MailComposer unit tests', function() {
             compiler._createAlternative.restore();
         });
 
-        it('should create Related structure with html and cid attachment', function() {
+        it('should create Related structure with html and cid attachment', function () {
             let data = {
                 html: 'def',
                 attachments: [
@@ -198,7 +198,7 @@ describe('MailComposer unit tests', function() {
             compiler._createRelated.restore();
         });
 
-        it('should create content node with only text', function() {
+        it('should create content node with only text', function () {
             let data = {
                 text: 'def'
             };
@@ -210,7 +210,7 @@ describe('MailComposer unit tests', function() {
             compiler._createContentNode.restore();
         });
 
-        it('should create content node with only an attachment', function() {
+        it('should create content node with only an attachment', function () {
             let data = {
                 attachments: [
                     {
@@ -227,7 +227,7 @@ describe('MailComposer unit tests', function() {
             compiler._createContentNode.restore();
         });
 
-        it('should create content node with encoded buffer', function() {
+        it('should create content node with encoded buffer', function () {
             let str = 'tere tere';
             let data = {
                 text: {
@@ -241,7 +241,7 @@ describe('MailComposer unit tests', function() {
             expect(compiler.message.content).to.deep.equal(Buffer.from(str));
         });
 
-        it('should create content node from data url', function() {
+        it('should create content node from data url', function () {
             let str = 'tere tere';
             let data = {
                 attachments: [
@@ -258,7 +258,37 @@ describe('MailComposer unit tests', function() {
             expect(compiler.mail.attachments[0].contentType).to.equal('image/png');
         });
 
-        it('should create the same output', function(done) {
+        it('should not treat invalid content-type as multipart', function () {
+            let data = {
+                from: {
+                    name: 'sender name',
+                    address: 'sender@example.com'
+                },
+                to: [
+                    {
+                        address: 'andris@ethereal.email'
+                    }
+                ],
+                subject: 'Hello world!',
+                text: 'Test message',
+                attachments: [
+                    {
+                        contentType: 'other',
+                        content: Buffer.from('tere').toString('base64'),
+                        encoding: 'base64'
+                    }
+                ]
+            };
+
+            let compiler = new MailComposer(data);
+            let mail = compiler.compile();
+            let attachmentHeaders = mail.childNodes[1].buildHeaders();
+
+            // attachment header must not contain mutipart header
+            expect(attachmentHeaders.indexOf('boundary=')).to.lt(0);
+        });
+
+        it('should create the same output', function (done) {
             let data = {
                 text: 'abc',
                 html: 'def',
@@ -285,12 +315,12 @@ describe('MailComposer unit tests', function() {
                 'MIME-Version: 1.0\r\n' +
                 '\r\n' +
                 '----_NmP-test-Part_1\r\n' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'Content-Transfer-Encoding: 7bit\r\n' +
                 '\r\n' +
                 'abc\r\n' +
                 '----_NmP-test-Part_1\r\n' +
-                'Content-Type: text/html\r\n' +
+                'Content-Type: text/html; charset=utf-8\r\n' +
                 'Content-Transfer-Encoding: 7bit\r\n' +
                 '\r\n' +
                 'def\r\n' +
@@ -298,14 +328,14 @@ describe('MailComposer unit tests', function() {
 
             let mail = new MailComposer(data).compile();
             expect(mail.messageId()).to.equal('<zzzzzz>');
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should use raw input for the message', function(done) {
+        it('should use raw input for the message', function (done) {
             let data = {
                 raw: 'test test test\r\n',
                 envelope: {
@@ -317,7 +347,7 @@ describe('MailComposer unit tests', function() {
             let expected = 'test test test\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(mail.getEnvelope()).to.deep.equal({
                     from: 'deamon@kreata.ee',
@@ -328,7 +358,7 @@ describe('MailComposer unit tests', function() {
             });
         });
 
-        it('should use raw input for different parts', function(done) {
+        it('should use raw input for different parts', function (done) {
             let data = {
                 from: 'test1@example.com',
                 to: 'test2@example.com',
@@ -393,14 +423,14 @@ describe('MailComposer unit tests', function() {
                 '----_NmP-test-Part_1--\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should discard BCC', function(done) {
+        it('should discard BCC', function (done) {
             let data = {
                 from: 'test1@example.com',
                 to: 'test2@example.com',
@@ -412,7 +442,7 @@ describe('MailComposer unit tests', function() {
 
             let expected =
                 '' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'From: test1@example.com\r\n' +
                 'To: test2@example.com\r\n' +
                 'Message-ID: <zzzzzz>\r\n' +
@@ -423,14 +453,14 @@ describe('MailComposer unit tests', function() {
                 'def\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should autodetect text encoding', function(done) {
+        it('should autodetect text encoding', function (done) {
             let data = {
                 from: 'ÄÄÄÄ test1@example.com',
                 to: 'AAAÄ test2@example.com',
@@ -455,14 +485,14 @@ describe('MailComposer unit tests', function() {
                 'def =C3=84=C3=84=C3=84=C3=84 foo AAA=C3=84\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should use quoted-printable text encoding', function(done) {
+        it('should use quoted-printable text encoding', function (done) {
             let data = {
                 from: 'ÄÄÄÄ test1@example.com',
                 to: 'AAAÄ test2@example.com',
@@ -488,14 +518,14 @@ describe('MailComposer unit tests', function() {
                 'def =C3=84=C3=84=C3=84=C3=84 foo AAA=C3=84\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should use base64 text encoding', function(done) {
+        it('should use base64 text encoding', function (done) {
             let data = {
                 from: 'ÄÄÄÄ test1@example.com',
                 to: 'AAAÄ test2@example.com',
@@ -520,14 +550,14 @@ describe('MailComposer unit tests', function() {
                 'ZGVmIMOEw4TDhMOEIGZvbyBBQUHDhA==\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should keep BCC', function(done) {
+        it('should keep BCC', function (done) {
             let data = {
                 from: 'test1@example.com',
                 to: 'test2@example.com',
@@ -539,7 +569,7 @@ describe('MailComposer unit tests', function() {
 
             let expected =
                 '' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'From: test1@example.com\r\n' +
                 'To: test2@example.com\r\n' +
                 'Bcc: test3@example.com\r\n' +
@@ -552,14 +582,14 @@ describe('MailComposer unit tests', function() {
 
             let mail = new MailComposer(data).compile();
             mail.keepBcc = true;
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should set headers for attachment', function(done) {
+        it('should set headers for attachment', function (done) {
             let data = {
                 text: 'abc',
                 baseBoundary: 'test',
@@ -586,7 +616,7 @@ describe('MailComposer unit tests', function() {
                 'MIME-Version: 1.0\r\n' +
                 '\r\n' +
                 '----_NmP-test-Part_1\r\n' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'Content-Transfer-Encoding: 7bit\r\n' +
                 '\r\n' +
                 'abc\r\n-' +
@@ -603,14 +633,57 @@ describe('MailComposer unit tests', function() {
                 '----_NmP-test-Part_1--\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should keep plaintext for attachment', function(done) {
+        it('should encode filename', function (done) {
+            let data = {
+                text: 'abc',
+                baseBoundary: 'test',
+                messageId: 'zzzzzz',
+                date: 'Sat, 21 Jun 2014 10:52:44 +0000',
+                attachments: [
+                    {
+                        content: 'test',
+                        filename: '"test" it is.txt'
+                    }
+                ]
+            };
+
+            let expected =
+                '' +
+                'Content-Type: multipart/mixed; boundary="--_NmP-test-Part_1"\r\n' +
+                'Message-ID: <zzzzzz>\r\n' +
+                'Date: Sat, 21 Jun 2014 10:52:44 +0000\r\n' +
+                'MIME-Version: 1.0\r\n' +
+                '\r\n' +
+                '----_NmP-test-Part_1\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
+                'Content-Transfer-Encoding: 7bit\r\n' +
+                '\r\n' +
+                'abc\r\n-' +
+                '---_NmP-test-Part_1\r\n' +
+                'Content-Type: text/plain; name="=?UTF-8?Q?=22test=22_it_is=2Etxt?="\r\n' +
+                'Content-Transfer-Encoding: base64\r\n' +
+                'Content-Disposition: attachment;\r\n' +
+                " filename*0*=utf-8''%22test%22%20it%20is.txt\r\n" +
+                '\r\n' +
+                'dGVzdA==\r\n' +
+                '----_NmP-test-Part_1--\r\n';
+
+            let mail = new MailComposer(data).compile();
+            mail.build(function (err, message) {
+                expect(err).to.not.exist;
+                expect(message.toString()).to.equal(expected);
+                done();
+            });
+        });
+
+        it('should keep plaintext for attachment', function (done) {
             let data = {
                 text: 'abc',
                 baseBoundary: 'test',
@@ -638,7 +711,7 @@ describe('MailComposer unit tests', function() {
                 'MIME-Version: 1.0\r\n' +
                 '\r\n' +
                 '----_NmP-test-Part_1\r\n' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'Content-Transfer-Encoding: 7bit\r\n' +
                 '\r\n' +
                 'abc\r\n-' +
@@ -655,14 +728,14 @@ describe('MailComposer unit tests', function() {
                 '----_NmP-test-Part_1--\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should ignore attachment filename', function(done) {
+        it('should ignore attachment filename', function (done) {
             let data = {
                 text: 'abc',
                 baseBoundary: 'test',
@@ -688,7 +761,7 @@ describe('MailComposer unit tests', function() {
                 'MIME-Version: 1.0\r\n' +
                 '\r\n' +
                 '----_NmP-test-Part_1\r\n' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'Content-Transfer-Encoding: 7bit\r\n' +
                 '\r\n' +
                 'abc\r\n' +
@@ -707,14 +780,14 @@ describe('MailComposer unit tests', function() {
                 '----_NmP-test-Part_1--\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should add ical alternative', function(done) {
+        it('should add ical alternative', function (done) {
             let data = {
                 from: 'test1@example.com',
                 to: 'test2@example.com',
@@ -743,7 +816,7 @@ describe('MailComposer unit tests', function() {
                 'Content-Type: multipart/alternative; boundary="--_NmP-test-Part_2"\r\n' +
                 '\r\n' +
                 '----_NmP-test-Part_2\r\n' +
-                'Content-Type: text/plain\r\n' +
+                'Content-Type: text/plain; charset=utf-8\r\n' +
                 'Content-Transfer-Encoding: 7bit\r\n' +
                 '\r\n' +
                 'def\r\n' +
@@ -763,14 +836,14 @@ describe('MailComposer unit tests', function() {
                 '----_NmP-test-Part_1--\r\n';
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.equal(expected);
                 done();
             });
         });
 
-        it('should use load attachment from file', function(done) {
+        it('should use load attachment from file', function (done) {
             let data = {
                 text: 'abc',
                 attachments: [
@@ -781,14 +854,14 @@ describe('MailComposer unit tests', function() {
             };
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.not.exist;
                 expect(message.toString()).to.include('w7VrdmEK');
                 done();
             });
         });
 
-        it('should not load attachment from file', function(done) {
+        it('should not load attachment from file', function (done) {
             let data = {
                 text: 'abc',
                 attachments: [
@@ -800,7 +873,7 @@ describe('MailComposer unit tests', function() {
             };
 
             let mail = new MailComposer(data).compile();
-            mail.build(function(err, message) {
+            mail.build(function (err, message) {
                 expect(err).to.exist;
                 expect(message).to.not.exist;
                 done();
